@@ -1,5 +1,6 @@
-from typing import Dict, List, Optional, Type
+from typing import Dict, List, Optional, Tuple, Type, Union
 
+import astropy.units as u
 import lightweaver as lw
 import lightweaver.wittmann as witt
 import numpy as np
@@ -79,6 +80,16 @@ class IsoPromModel(PromModel):
         When running a grid of models, consider creating a
         `TabulatedPromBcProvider` using `compute_falc_bc_ctx` and `tabulate_bc`,
         since the default performs quite a few extra RT calculations.
+    add_vertical_ray : bool, optional
+        Whether to add a non-weighted vertical ray to the model. This doesn't
+        participate in energy balance but allows for directly extracting mu=1,
+        without needing to do a boundary condition. Default: False. Equivalent to
+        `add_extra_mus={"muz": [1.0], "mux": [0.0]}`.
+    add_extra_rays : dict, optional
+        Extra rays to add to the quadrature along which the solution should be
+        sampled. These rays will not have a weight for integration, but can
+        serve as output for different viewing angles. Should be a dict with keys
+        `muz` and `mux` as iterables.
     """
 
     def __init__(
@@ -103,6 +114,8 @@ class IsoPromModel(PromModel):
         BcType: Optional[Type[PromBc]] = None,
         bc_kwargs=None,
         bc_provider=None,
+        add_vertical_ray: bool = False,
+        add_extra_rays: Dict[str, Union[List[float], Tuple[float], np.ndarray]] = None,
     ):
         self.projection = projection
         if projection not in ["prominence", "filament"]:
@@ -128,6 +141,13 @@ class IsoPromModel(PromModel):
                 "Cannot set both vrad and vlos for a filament. (Just set one of the two)."
             )
 
+        if add_vertical_ray and add_extra_rays is not None:
+            raise ValueError(
+                "Cannot provide extra_rays and set extra_rays dict simultaneously."
+            )
+        if add_vertical_ray:
+            add_extra_rays = {"muz": [1.0], "mux": [0.0]}
+
         if projection == "filament" and vrad is not None and vlos is None:
             vlos = vrad
             self.vlos = vrad
@@ -172,11 +192,10 @@ class IsoPromModel(PromModel):
                 witt.BK * temperature
             )
         nHTot = (
-            rho
-            / (lw.CM_TO_M**3 / lw.G_TO_KG)
+            (rho << u.Unit("g / cm3")).to("kg / m3").value
             / (lw.Amu * lw.DefaultAtomicAbundance.massPerH)
         )
-        ne /= lw.CM_TO_M**3
+        ne = (ne << u.Unit("cm-3")).to("m-3").value
         # NOTE(cmo): CGS Ends Here
 
         lower_bc = BcType(projection, bc_provider, altitude, "lower", **bc_kwargs)
@@ -200,6 +219,14 @@ class IsoPromModel(PromModel):
             upperBc=upper_bc,
         )
         self.atmos.quadrature(Nrays)
+        if add_extra_rays is not None:
+            extra_wmu = np.zeros_like(add_extra_rays["muz"])
+            self.atmos.rays(
+                muz=np.concatenate([self.atmos.muz, add_extra_rays["muz"]]),
+                mux=np.concatenate([self.atmos.mux, add_extra_rays["mux"]]),
+                wmu=np.concatenate([self.atmos.wmu, extra_wmu])
+            )
+
 
         self.rad_set = lw.RadiativeSet(atomic_models)
         self.rad_set.set_active(*active_atoms)
@@ -274,7 +301,7 @@ class IsoPromModel(PromModel):
                 )
                 if printNow:
                     print(
-                        f"    nHTotError: {np.max(np.abs(nHTotCorrection / self.atmos.nHTot))}"
+                        f"    nHTotError: {np.max(np.abs(nHTotCorrection / self.atmos.nHTot)):6.4e}"
                     )
                 self.atmos.ne[:] += (
                     nHTotCorrection * self.eq_pops["H"][-1] / self.atmos.nHTot
